from typing import cast

import psycopg
from flask import Flask, Response, g, jsonify, request

from pgqueuer.db import SyncPsycopgDriver, dsn
from pgqueuer.queries import SyncQueries

app = Flask(__name__)


def get_driver() -> SyncPsycopgDriver:
    if "driver" not in g:
        conn = psycopg.connect(dsn(), autocommit=True)
        g.driver = SyncPsycopgDriver(conn)

    return cast(SyncPsycopgDriver, g.driver)


@app.teardown_appcontext
def teardown_db(exception: BaseException | None) -> None:
    driver = cast(SyncPsycopgDriver | None, g.pop("driver", None))

    if driver is not None:
        driver._connection.close()


@app.route("/enqueue", methods=["POST"])
def enqueue() -> Response:
    queries = SyncQueries(get_driver())
    data = request.get_json(force=True)
    entrypoint = data.get("entrypoint", "default")
    payload = data.get("payload")
    priority = int(data.get("priority", 0))

    job_ids = queries.enqueue(entrypoint, payload.encode() if payload else None, priority)
    return jsonify({"job_ids": [str(jid) for jid in job_ids]})


@app.route("/queue_size")
def queue_size() -> Response:
    queries = SyncQueries(get_driver())
    stats = queries.queue_size()
    return jsonify(
        [
            {
                "entrypoint": s.entrypoint,
                "priority": s.priority,
                "status": s.status,
                "count": s.count,
            }
            for s in stats
        ]
    )


if __name__ == "__main__":
    app.run(debug=True)



from typing import AsyncGenerator

import asyncpg
from fastapi import Depends, FastAPI

from pgqueuer.db import AsyncpgDriver, Driver
from pgqueuer.queries import Queries

app = FastAPI()


async def get_driver() -> AsyncGenerator[Driver, None]:
    conn = await asyncpg.connect()
    try:
        yield AsyncpgDriver(conn)
    finally:
        await conn.close()


@app.post("/enqueue")
async def enqueue_job(
    entrypoint: str,
    payload: str,
    priority: int = 0,
    driver: AsyncpgDriver = Depends(get_driver),
) -> dict:
    queries = Queries(driver)
    ids = await queries.enqueue(entrypoint, payload.encode(), priority)
    return {"job_ids": ids}


@app.get("/queue-size")
async def get_queue_size(
    driver: AsyncpgDriver = Depends(get_driver),
) -> list:
    queries = Queries(driver)
    stats = await queries.queue_size()
    return [
        {
            "entrypoint": s.entrypoint,
            "priority": s.priority,
            "status": s.status,
            "count": s.count,
        }
        for s in stats
    ]


if __name__ == "__main__":
    import uvicorn

    uvicorn.run(app)



import asyncio
from typing import AsyncGenerator, Generator

import asyncpg
import psycopg
import pytest
import uvloop

from pgqueuer.db import AsyncpgDriver, Driver, SyncPsycopgDriver, dsn
from pgqueuer.queries import Queries


@pytest.fixture(scope="session", autouse=True)
def event_loop_policy() -> uvloop.EventLoopPolicy:
    return uvloop.EventLoopPolicy()


@pytest.fixture(scope="function")
async def apgdriver() -> AsyncGenerator[AsyncpgDriver, None]:
    conn = await asyncpg.connect(dsn=dsn())
    try:
        yield AsyncpgDriver(conn)
    finally:
        await conn.close()


@pytest.fixture(scope="function")
def pgdriver() -> Generator[SyncPsycopgDriver, None, None]:
    conn = psycopg.connect(dsn())
    try:
        yield SyncPsycopgDriver(conn)
    finally:
        conn.close()


@pytest.fixture(scope="function", autouse=True)
async def truncate_tables(apgdriver: Driver) -> None:
    await asyncio.gather(
        Queries(apgdriver).clear_queue_log(),
        Queries(apgdriver).clear_queue(),
        Queries(apgdriver).clear_schedule(),
        Queries(apgdriver).clear_statistics_log(),
    )
